rm(list=ls(all=TRUE))

library(tidyverse)
library(pwr)
library(estimatr)
library(stargazer)
library(xtable)

# set working directory
# setwd("~/Dropbox/Yale/VoterVoting/Shared_2016VoterVoting/BPP_replication/")

##================================##
## Read in data
##================================##

d <- read.csv("data.csv", header = TRUE, stringsAsFactors = FALSE, strip.white = TRUE)

##================================##
## Define helper functions
##================================##

## Helper function to estimate group proportions, standard errors, n's
group_props <- function(data, sampdef) {
  out <- data %>%
    group_by(treat) %>%
    summarise(prop = mean(vote_16g),
              se = sqrt( (prop * ( 1 - prop)) / n()),
              n = n()) %>%
    mutate(sample = sampdef)
  return(out)
}

## Helper function to conduct nonparametric difference-in-proportions analysis
##  (Noun - Placebo; Verb - Placebo; and Noun - Verb)
## DEFAULT: one-sided test that group A proportion > group B proportion (e.g., Noun > Verb)
## NOTE: for a two-sided test, specify alt = "two.sided" in function call
prop_test_pairwise <- function(data, sampdef, alt = NULL) {
  
  if ( is.null(alt) ) {
    alt <- "greater"
  }
  
  # noun - placebo
  proptest_est <- prop.test(x = with(data, table(treat, vote_16g))[1:2,2], 
                            n = with(data, table(treat))[1:2], correct = FALSE, alternative = alt)
  
  g1 <- "noun"; g2 <- "placebo"
  p1         <- proptest_est$estimate[1]
  p2         <- proptest_est$estimate[2]
  n1         <- with(data, table(treat))[1] # noun group n
  n2         <- with(data, table(treat))[2] # placebo group n
  diff_est   <- p1 - p2
  diff_se    <- sqrt( ((p1 * (1 - p1))/n1 ) + ((p2 * (1 - p2))/n2) )
  diff_z     <- diff_est / diff_se
  diff_p     <- proptest_est$p.value
  diff_lci   <- proptest_est$conf.int[1]
  diff_uci   <- proptest_est$conf.int[2]
  chisq_stat <- proptest_est$statistic
  chisq_df   <- proptest_est$parameter
  cohens_h   <- pwr::ES.h(p1 = p1, p2 = p2)  # cohen's h
  out_np <- c(g1, g2, p1, p2, n1, n2, 
              diff_est, diff_se, diff_z, diff_p, diff_lci, diff_uci, 
              chisq_stat, chisq_df, cohens_h, "noun - placebo")
  names(out_np) <- c("group1", "group2", "prop1", "prop2", "n1", "n2", 
                     "diff_est", "diff_se", "diff_z", "diff_p", "diff_lci", "diff_uci", 
                     "chisq_stat", "chisq_df", "cohens_h", "comparison")
  
  # verb - placebo
  proptest_est <- prop.test(x = with(data, table(treat, vote_16g))[c(3,2),2], 
                            n = with(data, table(treat))[c(3,2)], correct = FALSE, alternative = alt)
  
  g1 <- "verb"; g2 <- "placebo"
  p1         <- proptest_est$estimate[1]
  p2         <- proptest_est$estimate[2]
  n1         <- with(data, table(treat))[3] # verb group n
  n2         <- with(data, table(treat))[2] # placebo group n
  diff_est   <- p1 - p2
  diff_se    <- sqrt( ((p1 * (1 - p1))/n1 ) + ((p2 * (1 - p2))/n2) )
  diff_z     <- diff_est / diff_se
  diff_p     <- proptest_est$p.value
  diff_lci   <- proptest_est$conf.int[1]
  diff_uci   <- proptest_est$conf.int[2]
  chisq_stat <- proptest_est$statistic
  chisq_df   <- proptest_est$parameter
  cohens_h   <- pwr::ES.h(p1 = p1, p2 = p2)  # cohen's h
  out_vp     <- c(g1, g2, p1, p2, n1, n2, 
                  diff_est, diff_se, diff_z, diff_p, diff_lci, diff_uci, 
                  chisq_stat, chisq_df, cohens_h, "verb - placebo")
  names(out_vp) <- c("group1", "group2", "prop1", "prop2", "n1", "n2", 
                     "diff_est", "diff_se", "diff_z", "diff_p", "diff_lci", "diff_uci", 
                     "chisq_stat", "chisq_df", "cohens_h", "comparison")
  
  # noun - verb
  proptest_est <- prop.test(x = with(data, table(treat, vote_16g))[c(1,3),2],
                            n = with(data, table(treat))[c(1,3)], correct = FALSE, alternative = alt)
  
  g1 <- "noun"; g2 <- "verb"
  p1         <- proptest_est$estimate[1]
  p2         <- proptest_est$estimate[2]
  n1         <- with(data, table(treat))[1] # noun group n
  n2         <- with(data, table(treat))[3] # verb group n
  diff_est   <- p1 - p2
  diff_se    <- sqrt( ((p1 * (1 - p1))/n1 ) + ((p2 * (1 - p2))/n2) )
  diff_z     <- diff_est / diff_se
  diff_p     <- proptest_est$p.value
  diff_lci   <- proptest_est$conf.int[1]
  diff_uci   <- proptest_est$conf.int[2]
  chisq_stat <- proptest_est$statistic
  chisq_df   <- proptest_est$parameter
  cohens_h   <- pwr::ES.h(p1 = p1, p2 = p2)  # cohen's h
  out_nv     <- c(g1, g2, p1, p2, n1, n2, 
                  diff_est, diff_se, diff_z, diff_p, diff_lci, diff_uci, 
                  chisq_stat, chisq_df, cohens_h, "noun - verb")
  names(out_nv) <- c("group1", "group2", "prop1", "prop2", "n1", "n2", 
                     "diff_est", "diff_se", "diff_z", "diff_p", "diff_lci", "diff_uci", 
                     "chisq_stat", "chisq_df", "cohens_h", "comparison")
  
  # stack and reshape output
  out <- bind_rows(out_np, out_vp, out_nv) %>%
    gather(param, value, -comparison, -group1, -group2) %>%
    mutate(value = as.numeric(value)) %>%
    spread(param, value) %>%
    mutate(sample = sampdef) %>%
    select(sample, comparison, group1, group2, diff_est, diff_se, diff_z, diff_p, diff_lci, diff_uci, 
           prop1, prop2, n1, n2, chisq_df, chisq_stat, cohens_h)
  
  return(out)
}


##================================##
## Table 1: Nonparametric Analyses
##================================##

## CALCULATE GROUP PROPORTIONS, STANDARD ERRORS, N

# Entire sample
gp_all <- list(group_props(data = d, sampdef = "Entire sample"))

# By state
states <- names(table(d$state))
gp_bystate <- lapply(states, function(J) group_props(data = d[d$state == J,], 
                                                     sampdef = paste(J, "subsample")))

# By high or low electoral competition state
comp <- names(table(d$competitive))
gp_bycomp <- lapply(comp, function(J) group_props(data = d[d$competitive == J,], 
                                                  sampdef = paste(J, "electoral competition subsample")))

## ESTIMATE NONPARAMETRIC DIFFERENCE IN PROPORTIONS FOR THE FOLLOWING PAIRWISE COMPARISONS
## (1) Noun - Placebo
## (2) Verb - Placebo
## (3) Noun - Verb

# Entire sample
dip_all <- list(prop_test_pairwise(data = d, sampdef = "Entire sample", alt = "greater"))

# By state
dip_bystate <- lapply(states, function(J) prop_test_pairwise(data = d[d$state == J,], 
                                                             sampdef = paste(J, "subsample"),
                                                             alt = "greater"))

# By high or low electoral competition state
dip_bycomp <- lapply(comp, function(J) prop_test_pairwise(data = d[d$competitive == J,], 
                                                          sampdef = paste(J, "electoral competition subsample"),
                                                          alt = "greater"))

## MAKE TABLE 1

# Group proportions (left panel)
tab1_gp <- c(gp_all, gp_bystate, gp_bycomp)
tab1_gp <- do.call(bind_rows, tab1_gp) %>%
  select(sample, treat, prop, se, n)

tab1_left <- tab1_gp %>%
  gather(param, key, -sample, -treat) %>%
  mutate(treat_param = paste0(treat, "-", param)) %>%
  select(-treat, -param) %>%
  spread(treat_param, key) %>%
  select(sample, `Placebo-prop`, `Placebo-se`, `Placebo-n`,
         `Noun-prop`, `Noun-se`, `Noun-n`,
         `Verb-prop`, `Verb-se`, `Verb-n`) %>%
  mutate(sample = factor(sample, levels = c("Entire sample",
                                            paste(states, "subsample"),
                                            paste(comp, "electoral competition subsample")
  ))) %>%
  arrange(sample)

# Diff-in-proportions (right panel)
tab1_dip <- c(dip_all, dip_bystate, dip_bycomp)
tab1_dip <- do.call(bind_rows, tab1_dip)

tab1_right <- tab1_dip %>%
  select(sample, comparison, diff_est, diff_se) %>%
  gather(param, value, -sample, -comparison) %>%
  mutate(comparison = gsub(" ", "", comparison, fixed=TRUE),
         param = gsub("diff_", "", param, fixed=TRUE),
         comp_param = paste(comparison, param, sep="-")) %>%
  select(-comparison, -param) %>%
  spread(comp_param, value) %>%
  select(sample, `noun-placebo-est`, `noun-placebo-se`,
         `verb-placebo-est`, `verb-placebo-se`,
         `noun-verb-est`, `noun-verb-se`) %>%
  mutate(sample = factor(sample, levels = c("Entire sample",
                                            paste(states, "subsample"),
                                            paste(comp, "electoral competition subsample")
  ))) %>%
  arrange(sample)

# Join together to make Table 1
table2 <- left_join(tab1_left, tab1_right, by="sample") %>% as.data.frame()

# Round proportion, SE, and differenc estimates to 3 decimals
for(i in which(grepl("-prop", names(table2)) | grepl("-est", names(table2)) | grepl("-se", names(table2)))){
  table2[,i] <- sprintf("%0.3f", round(table2[,i], 3))
}

# Add parentheses around SE
for(i in which(grepl("-se", names(table2)))) {
  table2[,i] <- paste0("(", table2[,i],")")
}

# Convert n's to character
for(i in which(grepl("-n", names(table2)))) {
  table2[,i] <- as.character(table2[,i])
}


table2$sample <- as.character(table2$sample)
table2$sample <- ifelse(grepl("High electoral", table2$sample), paste(table2$sample, "(MI, OH, PA)"), table2$sample)
table2$sample <- ifelse(grepl("Low electoral", table2$sample), paste(table2$sample, "(CA, CT, NJ, NY)"), table2$sample)

table2_tex <- print(xtable(table2,
                           caption = "Nonparametric Estimates of Turnout Rates in the 2016 General Election and Differences in Turnout Rates Between Treatment Arms", 
                           label = "np"), 
                    scalebox=.7, caption.placement = "top",
                    include.rownames=FALSE)

table2_tex <- unlist(strsplit(table2_tex, "\n"))
row_swaps <- rbind(c("\\begin{tabular}{llllllllllllllll}", "\\begin{tabular}{lrrr|rrr|rrr|rr|rr|rr}"),
                   c("sample & Placebo-prop & Placebo-se & Placebo-n & Noun-prop & Noun-se & Noun-n & Verb-prop & Verb-se & Verb-n & noun-placebo-est & noun-placebo-se & verb-placebo-est & verb-placebo-se & noun-verb-est & noun-verb-se \\\\ ",
                     "  & \\multicolumn{3}{c}{~} & \\multicolumn{3}{c}{~} & \\multicolumn{3}{c}{~} & \\multicolumn{2}{c}{Difference in}  & \\multicolumn{2}{c}{Difference in}  & \\multicolumn{2}{c}{Difference in} \\\\      & \\multicolumn{3}{c}{Turnout Rate in} & \\multicolumn{3}{c}{Turnout Rate in} & \\multicolumn{3}{c}{Turnout Rate in} & \\multicolumn{2}{c}{Proportions}  & \\multicolumn{2}{c}{Proportions}  & \\multicolumn{2}{c}{Proportions} \\\\      & \\multicolumn{3}{c}{Placebo Condition} & \\multicolumn{3}{c}{Noun Condition} & \\multicolumn{3}{c}{Verb Condition} & \\multicolumn{2}{c}{(Noun - Placebo)}  & \\multicolumn{2}{c}{(Verb - Placebo)}  & \\multicolumn{2}{c}{(Noun - Verb)} \\\\    \\cline{2-16} Sample & Prop & (SE) & N &  Prop & (SE) & N &  Prop & (SE) & N & Estimate & (SE) & Estimate & (SE) & Estimate & (SE) \\\\"))

for(i in 1:nrow(row_swaps)){
  table2_tex <- gsub(row_swaps[i,1], row_swaps[i,2], table2_tex, fixed=TRUE)
}

# Export tex table

cat(table2_tex, file="Table_2.tex", sep="\n")



