rm(list=ls(all=TRUE))

library(tidyverse)
library(pwr)
library(estimatr)
library(stargazer)
library(xtable)
library(effsize)

# set working directory
# setwd("~/Dropbox/Yale/VoterVoting/Shared_2016VoterVoting/BPP_replication/")

#----------------------------------------#
# read in data
#----------------------------------------#

d <- read.csv("data.csv", header = TRUE, stringsAsFactors = FALSE, strip.white = TRUE)

states <- names(table(d$state))
comp <- names(table(d$competitive))

##================================##
## Helper functions
##================================##

## Extract p-values from one-tailed tests
otp <- function(fit, variable){
  s <- summary(fit)
  pt(coef(s)[,3], s$df[2], lower=FALSE)[variable]
}

## Make three part table using output from stargazer (float=FALSE)
make3ptable <- function(texinput, caption, label, resizebox, notes) {
  out <- c("\\begin{table}[!ht]",
           "\\centering",
           paste0("\\caption{",caption,"}"),
           paste0("\\label{",label,"}"),
           paste0("\\resizebox{",resizebox,"\\textwidth}{!}{"),
           "\\begin{threeparttable}",
           texinput,
           "\\begin{tablenotes}",
           "\\item $^{*}$p$<$0.1; $^{**}$p$<$0.05; $^{***}$p$<$0.01, two-tailed t-tests unless otherwise specified.",
           paste0("\\item ", notes),
           "\\end{tablenotes}",
           "\\end{threeparttable}",
           "}",
           "\\end{table}"
  )
  return(out)
}

##================================##
## Table S4: Regression Analysis
##          by Electoral Competitiveness
##================================##

# Define main model specifications

main_spec <- "vote_16g ~ treat_noun + treat_verb"  # PLACEBO IS OMITTED CATEGORY
vref_spec <- "vote_16g ~ placebo + treat_noun"     # VERB IS OMITTED CATEGORY
covs <- "age + agesq + female + race + educ + pid7 + ideo5 + day + vote_16p + vote_16pp + state"
covs_nostate <- "age + agesq + female + race + educ + pid7 + ideo5 + day + vote_16p + vote_16pp"
main_spec_adj <- paste(main_spec, covs, sep=" + ")
vref_spec_adj <- paste(vref_spec, covs, sep= "+ ")
main_spec_adj_nostate <- paste(main_spec, covs_nostate, sep=" + ")
vref_spec_adj_nostate <- paste(vref_spec, covs_nostate, sep= "+ ")

# Estimate models (omitted category: placebo)

fit_all <- NULL
ctr <- 0
for(J in comp){
  
  ctr <- ctr + 1
  fit_all[[ctr]] <- lm(formula = as.formula(main_spec), data = d, subset = competitive == J)
  
  ctr <- ctr + 1
  fit_all[[ctr]] <- lm(formula = as.formula(main_spec_adj), data = d, subset = competitive == J)  
}

# Estimate models (omitted category: verb) -- to get Noun-Verb difference

fit_vref <- NULL
ctr <- 0
for(J in comp){
  ctr <- ctr + 1
  fit_vref[[ctr]] <- lm(formula = as.formula(vref_spec), data = d, subset = competitive == J)
  ctr <- ctr + 1
  fit_vref[[ctr]] <- lm(formula = as.formula(vref_spec_adj), data = d, subset = competitive == J)
}

diff_nv <- lapply(fit_vref, function(J){
  x <- tidy(J)
  x <- x[x$term == 'treat_noun', c('estimate', 'std.error')]
  return(x)
})

diff_nv <- do.call(bind_rows, diff_nv)

diff_nv_est <- sprintf("%0.3f", round(diff_nv[,1], 3))

diff_nv_se <- sprintf("%0.3f", round(diff_nv[,2], 3))
diff_nv_se <- paste0("(", diff_nv_se, ")")

# One-sided tests

# Noun - Verb
otp_nv <- unlist(lapply(fit_vref, function(J) otp(J, "treat_noun")))
# Noun - Placebo
otp_np <- unlist(lapply(fit_all, function(J) otp(J, "treat_noun")))
# Verb - Placebo
otp_vp <- unlist(lapply(fit_all, function(J) otp(J, "treat_verb")))

otp_nv <- sprintf("%0.3f", round(otp_nv, 3))
otp_np <- sprintf("%0.3f", round(otp_np, 3))
otp_vp <- sprintf("%0.3f", round(otp_vp, 3))

# Format and output table

tableS4_tex <- stargazer(fit_all,
                         keep = c("treat", "Constant"),
                         covariate.labels = c("Voter (Noun) Treatment (1=Yes, 0=No)",
                                              "Voting (Verb) Treatment (1=Yes, 0=No)",
                                              "Constant"),
                         dep.var.caption = "",
                         dep.var.labels.include = FALSE,
                         column.labels = rep(comp, each = 2),
                         keep.stat = c("n"),
                         add.lines = list(c("Difference: Noun-Verb", diff_nv_est),
                                          c("~", diff_nv_se),
                                          c("p-value, one-tailed t-test, $H_0$: Noun-Verb=0; $H_A$: Noun-Verb>0", otp_nv),
                                          c("p-value, one-tailed t-test, $H_0$: Noun-Placebo=0; $H_A$: Noun-Placebo>0", otp_np),
                                          c("p-value, one-tailed t-test, $H_0$: Verb-Placebo=0; $H_A$: Verb-Placebo>0", otp_vp),
                                          c("With covariates?", rep(c("No", "Yes"), length(comp)))),
                         omit.table.layout = "n",
                         float = FALSE)

tableS4_tex <- make3ptable(texinput = tableS4_tex,
                           caption = "Regression Estimates of the Effect of Noun and Verb Treatments on Turnout in the 2016 General Election, by Ex Ante Electoral Competitiveness",
                           label = "reg_comp",
                           resizebox = "1",
                           notes = c("The outcome variables is turnout in the 2016 general election (1=Yes, 0=No). Covariates included in the covariate adjusted specification include age, age squared divided by 100, gender, race, education, survey date, party identification, ideology, past turnout in the 2016 primary and presidential primary elections, and state fixed effects.")
)


# Save table

cat(tableS4_tex, file = "tableS4.tex", sep="\n")





#### Re-estimate to get DF, t, 95% ci; calculate cohen's d

# Estimate models (omitted category: placebo)

est_all_store <- NULL
for(K in comp){
  
  est_all <- list(lm(formula = as.formula(main_spec), data = d, subset = competitive == K),
                  lm(formula = as.formula(main_spec_adj), data = d, subset = competitive == K))
  
  as.character(est_all[[1]]$terms[[2]])
  
  est_all <- lapply(est_all, function(J) {
    x <- tidy(J)
    x$conf.low <- x$estimate - qt(.975, summary(J)$df[2]) * x$std.error
    x$conf.high <- x$estimate + qt(.975, summary(J)$df[2]) * x$std.error
    x$df <- summary(J)$df[2]
    x$outcome <- as.character(J$terms[[2]])
    x$comparison_group <- "placebo"
    x$competitive <- K
    return(x)
  })
  
  est_all[[1]]$model <- "unadjusted"
  est_all[[2]]$model <- "cov_adjusted"
  est_all_store[[K]] <- do.call(bind_rows, est_all)
  
}
est_all_store <- do.call(bind_rows, est_all_store)

# Estimate models (omitted category: verb) -- to get Noun-Verb difference

est_vref_store <- NULL

for(K in comp){
  
  est_vref <- list(lm(formula = as.formula(vref_spec), data = d, subset = competitive == K),
                   lm(formula = as.formula(vref_spec_adj), data = d, subset = competitive == K))
  
  est_vref <- lapply(est_vref, function(J) {
    x <- tidy(J)
    x$conf.low <- x$estimate - qt(.975, summary(J)$df[2]) * x$std.error
    x$conf.high <- x$estimate + qt(.975, summary(J)$df[2]) * x$std.error
    x$df <- summary(J)$df[2]
    x$outcome <- as.character(J$terms[[2]])
    x$comparison_group <- "verb"
    x$competitive <- K
    return(x)
  })
  
  est_vref[[1]]$model <- "unadjusted"
  est_vref[[2]]$model <- "cov_adjusted"
  est_vref_store[[K]] <- do.call(bind_rows, est_vref)
  
}

est_vref_store <- do.call(bind_rows, est_vref_store)



# Stack estimation output

est_out <- bind_rows(est_all_store, 
                     est_vref_store) %>%
  mutate(conf.low.1t = estimate - qt(.95, df) * std.error)

vars <- c("estimate", "std.error", "statistic", "p.value", "conf.low", "conf.high", "conf.low.1t")
for(K in vars) est_out[[K]] <- sprintf("%0.3f", round(est_out[[K]], 3))

write.csv(est_out %>%
            filter(term %in% c("(Intercept)", "treat_noun", "treat_verb")), 
          "estout_reg_comp.csv", row.names=FALSE)


# cohen's d

est_cohen_d <- bind_rows(est_all_store[est_all_store$term %in% c("treat_noun", "treat_verb"),],
                         est_vref_store[est_vref_store$term %in% c("treat_noun", "treat_verb"),]) %>%
  mutate(N = NA)

for(K in comp) {
  est_cohen_d <- est_cohen_d %>%
    mutate(N = ifelse(competitive == K & comparison_group == "placebo" & term == "treat_noun", nrow(d[d$treat %in% c("Noun", "Placebo") & d$competitive == K,]) , N),
           N = ifelse(competitive == K & comparison_group == "placebo" & term == "treat_verb", nrow(d[d$treat %in% c("Verb","Placebo") & d$competitive == K,]) , N),
           N = ifelse(competitive == K & comparison_group == "verb" & term == "treat_noun", nrow(d[d$treat %in% c("Noun","Verb") & d$competitive == K,]) , N))
}

est_cohen_d <- est_cohen_d %>%
  mutate(qi = paste0(gsub("treat_", "", term, fixed=TRUE),"-", comparison_group),
         cohen_d = statistic / sqrt(N)) %>%
  select(qi, competitive, model, statistic, N, cohen_d)

vars <- c("statistic", "cohen_d")
for(K in vars) est_cohen_d[[K]] <- sprintf("%0.3f", round(est_cohen_d[[K]], 3))

write.csv(est_cohen_d, "cohend_reg_comp.csv", row.names=FALSE)

